//+------------------------------------------------------------------+
//|                                 Normalized Volume Oscillator.mq4 |
//|                                                  Vadim Shumiloff |
//|                                                shumiloff@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Vadim Shumiloff"
#property link      "shumiloff@mail.ru"

/*--------------------------------------------------------------------
NVO_v5:

First of all, I wish to express grateful thanks to MissPips for the 
coding which was critical to enabling the "Crown Jewels" of this effort
....the real-time update of the current bar volume, and the percentage 
that volume is to the average bar volume for the averaging period.

This version of the Normaized Volume Oscillator allows user to display
below average volume bars, or not.  Note that by not showing the below 
average volume bars, this indicator can be 'dragged and dropped' upon 
the MT4 'Volumes' indicator, to provide emphasis.

Also, the previous graduations of 38.2/61.8 are now user changeable,
and have been changed to 33/66.

New text labels show the current bar volume and its percentage of the
average bar volume (for the selected averaging period).  These labels
can be turned ON/OFF by selecting true/false.  Also, the text part of
the labels can be moved left, away from the displayed values, by 
increasing the numerical input for "Widen_text_labels".  

These labels come in 'Full' and 'Brief' sizes.  The 'Brief' size is 
for when the user is showing two charts side-by-side on MT4, but is
frequently restoring them to full size for better viewing.  When the
chart is full-screen, then the 'Full' label can be automatically
displayed.  And when the chart is then restored down, the the 'Brief'
label can automatically be displayed to better fit the reduced width
of the chart.  The 'Brief_mode_WindowsBarsPerChart' integer input is
the control for the automatic downsizing operation.  If a chart is
downsided (split screen charts for example) and there results less 
bars on the chart than this integer input, then the 'Brief' labels 
will display automatically if the 'Automatic_Full_to_Brief_mode?' 
is set to "true".

Be aware that a different integer input may be required for proper 
automatic operation of this function, depending upon the chart 'zoom'
mode the user has selected.  There are 6 zoom modes.  For the two 
modes in the center of this group the following intergers may be best,
150 for smaller candle mode/75 for the larger candle mode.  The user
will have to experiment to find the best integers to use when the
other zoom modes have been selected.

The user can 'force' the display of either size regardless of the
chart zoom setting.  To do this, select "false" for the input for
'Automatic_Full_to_Brief_mode?', and then select "true" or "false"
for the 'Manually_Size_Full?' input.
                                                       - Traderathome
----------------------------------------------------------------------
Volume Bar Colors Recommended:

  White CHART   Black CHART
  
  Blue          DarkSlateBlue   ZeroLine                                    
  Blue          DarkSlateBlue   Volumes below average
  Green         Green           Volumes in 1st graduation over average
  LimeGreen     LimeGreen       Volumes in 2nd graduation over average  
  Crimson       Yellow          Volumes in 3rd graduation over average
  Black         WhiteSmoke      Volumes greater than 100% over average               
---------------------------------------------------------------------*/

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 CLR_NONE           
#property indicator_color2 DarkSlateBlue   
#property indicator_color3 Green        
#property indicator_color4 LimeGreen   
#property indicator_color5 Yellow           
#property indicator_color6 White         
#property indicator_width1 1  
#property indicator_width2 2  
#property indicator_width3 2  
#property indicator_width4 2  
#property indicator_width5 2  
#property indicator_width6 2

/*---------------------------------------------------------------------
Label Colors Recommended:

  White CHART   Black CHART

  Purple        Magenta
  MediumBlue    CornflowerBlue   Volumes below average
  DarkGreen     Green            Volumes in 1st graduation over average
  Green         LimeGreen        Volumes in 2nd graduation over average
  Crimson       Yellow           Volumes in 3rd graduation over average
  Black         WhiteSmoke       Volumes greater than 100% over average
----------------------------------------------------------------------*/

//---- External Inputs
extern bool   Indicator_On?                   = true;
extern int    VolumePeriod                    = 10;
extern bool   ShowBelowAvgVolumes             = true;
extern string Need_2_numbers_dividing_range   = "0-100 into 3 parts (33&66, 25&50 etc)";
extern double Enter_Number_1                  = 33;
extern double Enter_Number_2                  = 66;
extern bool   Display_volume_text_labels?     = true;
extern int    Widen_text_labels               = 0;
extern color  VolumeCountTextColor            = Magenta;     
extern color  BelowAverageTextColor_0         = CornflowerBlue;
extern color  AboveAverageTextColor_1         = Green;  
extern color  AboveAverageTextColor_2         = LimeGreen; 
extern color  AboveAverageTextColor_3         = Yellow;     
extern color  AboveAverageTextColor_4         = WhiteSmoke;
extern int    Text_in_window_0123             = 3;
extern int    __Text_Horizontal_Indent        = 10;
extern int    __Text_Vertical_Indent          = 4;
extern int    FontStyle_Normal_Bold_12        = 1;
extern bool   Automatic_Full_to_Brief_mode?   = true;
extern bool   Manually_Size_Full?             = false;
extern int    Brief_mode_WindowBarsPerChart   = 150;

//---- Buffers & Other Inputs
double VolBufferH1[];
double VolBufferH2[];
double VolBufferH3[];
double VolBufferH4[];
double VolBufferH5[];
double VolBufferH6[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
   {
   IndicatorBuffers(6);
   
   //SetIndexBuffer(1, VolBufferH1);
   SetIndexStyle(0, DRAW_LINE);
   SetIndexBuffer(0, VolBufferH1);  
   
   //SetIndexBuffer(2, VolBufferH2);
   SetIndexStyle(1, DRAW_HISTOGRAM);
   SetIndexBuffer(1, VolBufferH2);

   //SetIndexBuffer(3, VolBufferH3);
   SetIndexStyle(2, DRAW_HISTOGRAM);
   SetIndexBuffer(2, VolBufferH3);
  
   //SetIndexBuffer(4, VolBufferH4);
   SetIndexStyle(3, DRAW_HISTOGRAM);
   SetIndexBuffer(3, VolBufferH4);

   //SetIndexBuffer(5, VolBufferH5);
   SetIndexStyle(4, DRAW_HISTOGRAM);
   SetIndexBuffer(4, VolBufferH5);

   //SetIndexBuffer(6, VolBufferH6);
   SetIndexStyle(5, DRAW_HISTOGRAM);
   SetIndexBuffer(5, VolBufferH6);
   
   //Indicator Window name and data labels 
   string short_name="NVO_v5  (" + VolumePeriod + ")   ";
   if (Indicator_On? == false)short_name=("NVO_v5  ("+VolumePeriod+")  Indicator is off.   ");
   IndicatorShortName(short_name);
   SetIndexLabel(0,  NULL);
   SetIndexLabel(1,  NULL); 
   SetIndexLabel(2,  NULL);
   SetIndexLabel(3,  NULL);
   SetIndexLabel(4,  NULL);
   SetIndexLabel(5,  NULL);
   SetIndexLabel(6,  NULL);
   SetIndexLabel(7,  NULL);

   return(0);
   }
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
   {
   int obj_total= ObjectsTotal();  
   for (int i= obj_total; i>=0; i--) 
      {
      string name= ObjectName(i);    
          if (StringSubstr(name,0,6)=="NVOtxt")
          {
          ObjectDelete(name);
          }
      }        
   return(0);
   }

//+-------------------------------------------------------------------+
//| Calculations body of program                                      |                                                        
//+-------------------------------------------------------------------+
int start()
   {
   if (Indicator_On? == false) {deinit();return(0);}
   
   string Spc = ""; for (int j = 0; j<Widen_text_labels; j++) {Spc = Spc + " ";}
   string spc = ""; if (FontStyle_Normal_Bold_12 == 1){spc = spc + " ";}
   int counted_bars = IndicatorCounted(); int i;
   double nv,nvo,nvo2; string Nvo = ""; string Vol = "";string BarVol=""; string Font = "Arial";
   if (FontStyle_Normal_Bold_12 == 2){Font = "Arial Bold";}
   
   for(i = Bars-1-IndicatorCounted(); i >= 0; i--) 
      {      
      VolBufferH1[i] = 0;
      VolBufferH2[i] = 0;
      VolBufferH3[i] = 0;
      VolBufferH4[i] = 0;
      VolBufferH5[i] = 0;
      VolBufferH6[i] = 0;
      
   if(Display_volume_text_labels?)
      {
      Vol = DoubleToStr(Volume[i], Digits-4);
      if(StringLen(Vol)==1){Vol = "          "+Vol;}
      if(StringLen(Vol)==2){Vol = "        "+Vol;}
      if(StringLen(Vol)==3){Vol = "      "+Vol;} 
      if(StringLen(Vol)==4){Vol = "    "+Vol;}   
      if(StringLen(Vol)==5){Vol = "  "+Vol;} 
      if(StringLen(Vol)==6){Vol = ""+Vol;}  
 
      if(FontStyle_Normal_Bold_12 == 1)
         {BarVol = "Vol Ct: "+Spc; Label4(BarVol+Vol+" v ", VolumeCountTextColor, 10, Font);}
         
         
      if(FontStyle_Normal_Bold_12 == 2) 
         {BarVol = "Vol Ct: "+Spc; Label4(BarVol+Vol+" v", VolumeCountTextColor, 10, Font);}
         
      if(Automatic_Full_to_Brief_mode? && WindowBarsPerChart() < Brief_mode_WindowBarsPerChart){Label4(Vol+spc+" v ",VolumeCountTextColor,9,Font);}
      if(Automatic_Full_to_Brief_mode? == false && Manually_Size_Full? == false){Label4(Vol+spc+" v ",VolumeCountTextColor,9,Font);}
      }
                       
      nv = NormalizedVolume(i);                               
      nvo = Volume[i]/nv*100 - 100;
      nvo2 = (Volume[i] /nv)*100-100;
                                    
      if (nvo<0) {Nvo = DoubleToStr(100-(nvo-(nvo*2)), Digits-4);}  //negative % changed to positive for text label      
      else {Nvo = DoubleToStr(nvo, Digits-4);}                      //the % already in positive form
      if (nvo>=0) Nvo = DoubleToStr(nvo+100, Digits-4);             //convert from normalized percent to real percent
      if(StringLen(Nvo)==1){Nvo = "      "+Nvo;}
      if(StringLen(Nvo)==2){Nvo = "    "+Nvo;}
      if(StringLen(Nvo)==3){Nvo = "  "+Nvo;}
      
      if (nvo<0)
         {
         VolBufferH2[i] = 0.25; if(ShowBelowAvgVolumes) {VolBufferH2[i] =nvo2;}        
         if(Display_volume_text_labels?)
            {
            Label("% Avg:     "+Spc+Nvo+"%",BelowAverageTextColor_0,10,Font);
            if(Automatic_Full_to_Brief_mode? && WindowBarsPerChart()<Brief_mode_WindowBarsPerChart) {Label(Nvo+" %",BelowAverageTextColor_0,9,Font);}
            if(Automatic_Full_to_Brief_mode? == false && Manually_Size_Full?==false) {Label(Nvo+" %",BelowAverageTextColor_0,9,Font);}
            }
         }
         
      else 
         {
         if (nvo<Enter_Number_1)
            {
            VolBufferH3[i]=nvo2;
            if(Display_volume_text_labels?)
               {
               Label("% Avg:     "+Spc+Nvo+"%",AboveAverageTextColor_1,10,Font);
               if(Automatic_Full_to_Brief_mode? && WindowBarsPerChart()<Brief_mode_WindowBarsPerChart) {Label(Nvo+" %",AboveAverageTextColor_1,9,Font);} 
               if(Automatic_Full_to_Brief_mode? == false && Manually_Size_Full?==false) {Label(Nvo+" %",AboveAverageTextColor_1,9,Font);}        
               }
             }
                
      else 
         {
         if (nvo<Enter_Number_2)
            {
            VolBufferH4[i]=nvo2;
            if(Display_volume_text_labels?)
               {
               Label("% Avg:     "+Spc+Nvo+"%",AboveAverageTextColor_2,10,Font);
               if(Automatic_Full_to_Brief_mode? && WindowBarsPerChart()<Brief_mode_WindowBarsPerChart) {Label(Nvo+" %",AboveAverageTextColor_2,9,Font);}
               if(Automatic_Full_to_Brief_mode? == false && Manually_Size_Full?==false) {Label(Nvo+" %",AboveAverageTextColor_2,9,Font);}
               }
            }
            
      else 
         {
         if (nvo<100)
            {
            VolBufferH5[i]=nvo2;
            if(Display_volume_text_labels?)
               {
               Label("% Avg:     "+Spc+Nvo+"%",AboveAverageTextColor_3,10,Font);
               if(Automatic_Full_to_Brief_mode? && WindowBarsPerChart()<Brief_mode_WindowBarsPerChart) {Label(Nvo+" %",AboveAverageTextColor_3,9,Font);}
               if(Automatic_Full_to_Brief_mode? == false && Manually_Size_Full?==false) {Label(Nvo+" %",AboveAverageTextColor_3,9,Font);}
               }
            }
             
      else
         {
         VolBufferH6[i]=nvo2;
         if(Display_volume_text_labels?)
            {
            Label("% Avg:     "+Spc+Nvo+"%",AboveAverageTextColor_4,10,Font);
            if(Automatic_Full_to_Brief_mode? && WindowBarsPerChart()<Brief_mode_WindowBarsPerChart) {Label(Nvo+" %",AboveAverageTextColor_4,9,Font);}
            if(Automatic_Full_to_Brief_mode? == false && Manually_Size_Full?==false) {Label(Nvo+" %",AboveAverageTextColor_4,9,Font);}
            }                
          }        
         }
        }
       }
         
      }
   return(0);   
   }     
        
//+-------------------------------------------------------------------+
//| Subroutine to draw volume count text label                        |                                                        
//+-------------------------------------------------------------------+
void Label4 (string text, color Color, int fontsize, string fontstyle)  
      {
      ObjectDelete("NVOtxt4");
      ObjectCreate("NVOtxt4", OBJ_LABEL, Text_in_window_0123, 0, 0);
      ObjectSet("NVOtxt4", OBJPROP_CORNER, 3);
      ObjectSet("NVOtxt4", OBJPROP_XDISTANCE, __Text_Horizontal_Indent);
      ObjectSet("NVOtxt4", OBJPROP_YDISTANCE,  __Text_Vertical_Indent+15);
      ObjectSet("NVOtxt4", OBJPROP_COLOR, Color);
      ObjectSetText("NVOtxt4", text, fontsize, fontstyle);    
      }
    
//+-------------------------------------------------------------------+
//| Subroutine to draw volume bar percentage text label               |                                                        
//+-------------------------------------------------------------------+
void Label(string text, color Color, int fontsize, string fontstyle)    
      {
      ObjectDelete("NVOtxt");
      ObjectCreate("NVOtxt", OBJ_LABEL, Text_in_window_0123, 0, 0);
      ObjectSet("NVOtxt", OBJPROP_CORNER, 3);
      ObjectSet("NVOtxt", OBJPROP_XDISTANCE, __Text_Horizontal_Indent);
      ObjectSet("NVOtxt", OBJPROP_YDISTANCE,  __Text_Vertical_Indent+0);
      ObjectSet("NVOtxt", OBJPROP_COLOR, Color);
      ObjectSetText("NVOtxt", text, fontsize, fontstyle);    
      }
   
//+-------------------------------------------------------------------+
//| Subroutine to compute normalized volume                           |                                                        
//+-------------------------------------------------------------------+
double NormalizedVolume(int i)
   {
   double nv = 0;
   for (int j = i; j < (i+VolumePeriod); j++) nv = nv + Volume[j];
   nv = nv / VolumePeriod;
   //return (Volume[i] / nv);
   return (nv);
   }
      
//+-------------------------------------------------------------------+
//| End of program                                                    |                                                        
//+-------------------------------------------------------------------+
   